# clausi.tcl
#
# "Clausi" style API for the National Instruments drivers
#
# 12/27/1999 bill@resuba.com
#
#    "Clausi" style API for the National Instruments drivers
#    Copyright (C) 1999  Bill Krimmel (bill@resuba.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

rename gpib nigpib

proc gpib {command args}  {
   switch -exact -- $command {
      find { return [llp_gpib::Find $args] }
      read { return [nigpib ibrd [lindex $args 0] [lrange $args 1 end]] }
      write { return [llp_gpib::write $args]}
      cmd { return [eval "nigpib ibcmd $args"] }
      ren { return [nigpib ibsre $llp_gpib::ibBoard [lrange $args 1 end]] }
      clear { return [eval "nigpib ibclr $args"] }
      onl { return [eval "nigpib ibonl $args"] }
      wait { return [eval "llp_gpib::Wait $args"] }
      rsp { return [eval "nigpib ibrsp $args"] }
      trg { return [eval "nigpib ibtrg $args"] }
      rsv { return [eval "nigpib ibrsv $llp_gpib::ibBoard [lrange $args 1 end]"] }
      sic { return 0}				;#ibsic??? 
      default {
         puts "Unrecognized Clausi gpib command: $command, $args"
      }
   }
}

namespace eval llp_gpib  {

# llp_gpib globals
   variable ConfigFile "/etc/gpib.conf"
   variable find 0  
   variable ibBoard -1
   variable ibBoardName "gpib0"
   variable ibGlobal

   proc write {args}  {
      variable ibGlobal

      set args [join $args " "]
      set ud [lindex $args 0]
      set data [join [lrange $args 1 end] " "]
      append data $ibGlobal($ud,eos)
      return [nigpib ibwrt $ud $data]
   }

   proc Find {name}  {
      variable ConfigFile
      variable find

      if [catch {open $ConfigFile r} fd]  {
         error "gpib find: Unable to open $fd"
      }

      if {$find == 0}  {
         ConfigBoard [findDevice $fd config]
         seek $fd 0 start
      }
      set dev [ConfigDevice [findDevice $fd $name]]
      close $fd
      incr find
      return $dev
   }

   
   proc findDevice {fd type}  {
      set state idle
      foreach line [split [read $fd] \n]  {
         regsub -all -- \{ $line \( line
         regsub -all -- \} $line \) line
         regsub -all -nocase -- \t+ $line " " line
         regsub -all -nocase -- " +" $line " " line
         foreach element $line  {
            switch -exact -- $state  {
               idle {
                  if {[string compare $element config] == 0 && \
	           [string compare $type config] == 0}  { 
                     set state OpenBrace
   	          }  else  {
	             if {[string compare $element device] == 0}  {
	                set state DeviceBrace
	             }
	          }
               } 

               OpenBrace  {
                  if {[string compare $element \( ] == 0}  {
                     set state DataName
                  }
               }

	       DeviceBrace  {
	          if {[string compare $element \(] == 0}  {
	             set state WaitName
	          }
	       }
		  
               DataName  {
                  if {[string compare $element \) ] == 0}  {
                     set state done
                  }  else  {
                        set temp $element
                        set state DataValue
	          }
               }
	    
               DataValue  {
   	          if {[string compare $element "\)"] == 0}  {
	             set state done
	             return $retval
	          }

                  if {[string compare $element "="] != 0}  {
                     lappend temp $element
                     lappend retval $temp
                     set state DataName
                  }
               }
	    
 	       WaitName  {
	          if {[string compare $element name] == 0}  {
	             set state Name
	          }
	       }

	       Name  {
	          if {[string compare $element "="] != 0}  {
	             if {[string compare $element $type] == 0}  {
	                set state DataName
	             }  else  {
                        set state idle
	             }
	          }
	       }
	       
	       done  {
	          return $retval
	       }
	    
            }       ;#switch state
         }          ;#foreach element
      }             ;#foreach line
   error "gpib find: $type not in gpib.conf"
   }

##################################################################################
   proc ConvertTimeout {ascii}  {
      set wad {NONE 10us 30us 100us 300us 1ms 3ms 10ms 30ms 100ms 300ms \
               1s 3s 10s 30s 100s 300s 1000s}
      if { [set x [lsearch $wad [string tolower $ascii]]] < 0}  {
         error "GPIB Error, $ascii not a valid timeout"
      }
      return $x
   }
##################################################################################
   proc YesNo {ascii}  {
      if { [string compare [string toupper [string index $ascii 0]] Y] == 0}  {
         return 1
      } else  {
         return 0
      }
   }

##################################################################################
   proc MakeHex {value}  {
      set x ""
      set c [string trimleft $value "0\\x"]
      set c [subst "\\x$c"]
      binary scan $c c retval
      return $retval
   }
##################################################################################

   proc ConfigBoard {goo}  {
      variable ibBoard
      variable ibBoardName
      variable ibGlobal
      set eos 0
      set ifc 0

      set ibBoard [nigpib ibfind $ibBoardName]

      nigpib ibrsc $ibBoard 1
      nigpib ibsic $ibBoard
      foreach pair $goo  {
         set name [lindex $pair 0]
         set value [lindex $pair 1]
         switch -exact -- $name  {

            pad   { nigpib ibpad $ibBoard $value }
            sad   { nigpib ibsad $ibBoard $value }
            timeout   { nigpib ibtmo $ibBoard [set tmo [ConvertTimeout $value]]}
            eos       { set eos [expr ($eos & 65280) + [MakeHex $value] ] }
            set-reos  { set eos [expr $eos + ([YesNo $value] * 1024)] }
            set-xeos  { set eos [expr $eos + ([YesNo $value] * 2048)] }
            set-bin   { set eos [expr $eos + ([YesNo $value] * 4096)] }
            set-ifc   { set ifc [YesNo $value] }

         } ;# switch name
      } ;# foreach goo
      set x [nigpib ibeos $ibBoard $eos]
      set ibGlobal(board,tmo) $tmo
      if {$ifc}  { nigpib ibsic $ibBoard }
   }
##################################################################################
   proc ConfigDevice {goo}  {
      variable ibBoard 
      variable ibGlobal

      set pad -1
      set sad 0
      set eot 1  ;# EOI on write, xeos???
      set eos 0
      set timeout -1

      foreach pair $goo  {
         set name [lindex $pair 0]
         set value [lindex $pair 1]
         switch -exact -- $name  {

            pad   { set pad $value }
            sad   { set sad $value }
            timeout   { set timeout [ConvertTimeout $value]}
            eos       { set eos [expr ($eos & 65280) + [set end [MakeHex $value] ] ] }
            set-reos  { set eos [expr $eos + ([YesNo $value] * 1024)] }
            set-xeos  { set eos [expr $eos + ([YesNo $value] * 2048)] }
            set-bin   { set eos [expr $eos + ([YesNo $value] * 4096)] }

         } ;# switch name
      } ;# foreach goo

      if {$pad < 0}  {error "ibFind: $name is not in gpib.conf"}
      if {$timeout < 0}  {set timeout $ibGlobal(board,tmo)}
      set ud [nigpib ibdev $ibBoard $pad $sad $timeout $eot $eos]
      set ibGlobal($ud,eos) [format "%c" $end]
      return $ud
   }
##################################################################################
   proc Wait {ud args}  {
      set mask 0
      foreach arg $args  {
         set x [lsearch -exact {cmpl cic atn srq timo rqs} $arg]
         set y [lindex {256 32 16 4096 16384 2048} $x]
         set mask [expr $mask + $y]
      }
      return [nigpib ibwait $ud $mask]
   }

} ;# namespace

